@@ -248,9 +248,6 @@ ARTsetup(void)
     hostcclass['-'] = 1;
     hostcclass['_'] = 1;
 
-    /* Also initialize the character class tables for message-IDs. */
-    InitializeMessageIDcclass();
-
     /* Build the header tree. */
     table = xmalloc(ARRAY_SIZE(ARTheaders) * sizeof(ARTHEADER *));
     for (i = 0; i < ARRAY_SIZE(ARTheaders); i++)
@@ -438,7 +435,7 @@ ARTheaderpcmp(const void *p1, const void *p2)
 /* Write an article using the storage api.  Put it together in memory and
    call out to the api. */
 static TOKEN
-ARTstore(CHANNEL *cp)
+ARTstore(CHANNEL *cp, bool filtered)
 {
     struct buffer *Article = &cp->In;
     ARTDATA *data = &cp->Data;
@@ -560,6 +557,7 @@ ARTstore(CHANNEL *cp)
     arth.arrived = (time_t) 0;
     arth.token = (TOKEN *) NULL;
     arth.expires = data->Expires;
+    arth.filtered = filtered;
     if (innconf->storeonxref) {
         arth.groups = data->Replic;
         arth.groupslen = data->ReplicLength;
@@ -567,6 +565,8 @@ ARTstore(CHANNEL *cp)
         arth.groups = HDR(HDR__NEWSGROUPS);
         arth.groupslen = HDR_LEN(HDR__NEWSGROUPS);
     }
+    arth.path = HDR(HDR__PATH);
+    arth.pathlen = HDR_LEN(HDR__PATH);
 
     SMerrno = SMERR_NOERROR;
     result = SMstore(arth);
@@ -880,6 +880,9 @@ ARTparseheader(CHANNEL *cp)
                     ARTcheckheader(cp, i - data->CurHeader);
                     data->CurHeader = i;
                 }
+                /* Remember that the line begins with a dot. */
+                if (bp->data[i] == '.')
+                    data->DotStuffedLines++;
             }
             if (bp->data[i] == '\r' && bp->data[i + 1] == '\n') {
                 cp->Next = i + 2;
@@ -1431,11 +1434,13 @@ ARTpoisongroup(char *name)
 }
 
 /*
-** Assign article numbers to the article and create the Xref header field.
-** If we end up not being able to write the article, we'll get "holes"
-** in the directory and active file.
+**  Assign article numbers to the article and create the Xref header field.
+**  If we end up not being able to write the article, we'll get "holes"
+**  in the directory and active file.
+**  Return false if a new article number could not be assigned (i.e. when
+**  it exceeds the maximum article number allowed in NNTP), and true otherwise.
 */
-static void
+static bool
 ARTassignnumbers(ARTDATA *data)
 {
     char *p, *q;
@@ -1456,6 +1461,11 @@ ARTassignnumbers(ARTDATA *data)
             continue;
 
         /* Bump the number. */
+        if (ngp->Last >= NNTP_MAXARTNUM) {
+            syslog(L_NOTICE, "%s full newsgroup %s (%lu articles)", LogName,
+                   ngp->Name, NNTP_MAXARTNUM);
+            return false;
+        }
         ngp->PostCount++;
         ngp->Last++;
         if (!FormatLong(ngp->LastString, (long) ngp->Last, ngp->Lastwidth)) {
@@ -1495,6 +1505,8 @@ ARTassignnumbers(ARTDATA *data)
     data->XrefLength = len + 2;
     data->Replic = q + 1;
     data->ReplicLength = len - (q + 1 - data->Xref);
+
+    return true;
 }
 
 /*
@@ -1554,6 +1566,12 @@ ARTxrefslave(ARTDATA *data)
         }
         *p = ':';
         ngp->Filenum = atol(p + 1);
+        if (ngp->Filenum > NNTP_MAXARTNUM) {
+            syslog(L_NOTICE, "%s full newsgroup %s (%lu articles)", LogName,
+                   ngp->Name, NNTP_MAXARTNUM);
+            return false;
+        }
+
         if (q != NULL)
             *q = c;
 
@@ -1679,7 +1697,7 @@ HashFeedMatch(HASHFEEDLIST *hf, char *MessageID)
 */
 static void
 ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
-             bool ControlStore, bool OverviewCreated, bool Filtered)
+             bool ControlStore, bool OverviewCreated, bool filtered)
 {
     HDRCONTENT *hc = data->HdrContent;
     SITE *sp, *funnel;
@@ -1800,7 +1818,7 @@ ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
         }
 
         /* Handle dontrejectfiltered. */
-        if (Filtered && sp->DropFiltered)
+        if (filtered && sp->DropFiltered)
             continue;
 
         /* Write that the site is getting it, and flag to send it. */
@@ -2540,7 +2558,16 @@ ARTpost(CHANNEL *cp)
             return false;
         }
     } else {
-        ARTassignnumbers(data);
+        if (!ARTassignnumbers(data)) {
+            snprintf(cp->Error, sizeof(cp->Error),
+                     "%d Newsgroup full (already %lu articles)",
+                     ihave ? NNTP_FAIL_IHAVE_REJECT
+                           : NNTP_FAIL_TAKETHIS_REJECT,
+                     NNTP_MAXARTNUM);
+            ARTlog(data, ART_REJECT, cp->Error);
+            ARTreject(REJECT_OTHER, cp);
+            return false;
+        }
     }
 
     /* Now we can file it. */
@@ -2552,7 +2579,7 @@ ARTpost(CHANNEL *cp)
     for (i = 0; (ngp = GroupPointers[i]) != NULL; i++)
         ngp->PostCount = 0;
 
-    token = ARTstore(cp);
+    token = ARTstore(cp, Filtered);
     /* Change trailing '\r\n' to '\0\n' of all system header fields. */
     for (i = 0; i < MAX_ARTHEADER; i++) {
         if (HDR_FOUND(i)) {
