@@ -126,6 +126,7 @@ static CCDISPATCH CCcommands[] = {
 };
 
 static void CCresetup(int s);
+static void CCnameWriteInfo(struct buffer *buffer, CHANNEL *cp);
 
 
 void
@@ -160,7 +161,7 @@ CCcurrmode(void)
     /* Server's mode. */
     switch (Mode) {
     default:
-        snprintf(buff, sizeof(buff), "Unknown %d", Mode);
+        snprintf(buff, sizeof(buff), "Unknown %u", Mode);
         return buff;
     case OMrunning:
         return "running";
@@ -186,7 +187,7 @@ CCsdnotify(void)
     /* Server's mode. */
     switch (Mode) {
     default:
-        buffer_append_sprintf(&CCreply, "Unknown %d", Mode);
+        buffer_append_sprintf(&CCreply, "Unknown %u", Mode);
         break;
     case OMrunning:
         buffer_append_sprintf(&CCreply, "running");
@@ -301,13 +302,13 @@ CCaddhist(char *av[])
         InndHisClose();
     if (strspn(av[1], DIGITS) != strlen(av[1]))
         return "1 Bad arrival date";
-    Data.Arrived = atol(av[1]);
+    Data.Arrived = atoll(av[1]);
     if (strspn(av[2], DIGITS) != strlen(av[2]))
         return "1 Bad expiration date";
-    Data.Expires = atol(av[2]);
+    Data.Expires = atoll(av[2]);
     if (strspn(av[3], DIGITS) != strlen(av[3]))
         return "1 Bad posted date";
-    Data.Posted = atol(av[3]);
+    Data.Posted = atoll(av[3]);
 
     /* Allow empty tokens, but not badly formatted tokens. */
     if (*av[4] != '\0' && !IsToken(av[4]))
@@ -883,7 +884,7 @@ CCmode(char *unused[] UNUSED)
     /* Server's mode. */
     switch (Mode) {
     default:
-        buffer_append_sprintf(&CCreply, "Unknown %d\n", Mode);
+        buffer_append_sprintf(&CCreply, "Unknown %u\n", Mode);
         break;
     case OMrunning:
         buffer_append_sprintf(&CCreply, "running\n");
@@ -905,11 +906,11 @@ CCmode(char *unused[] UNUSED)
         count++;
     buffer_append_sprintf(&CCreply,
                           "Parameters c %lu i %lu (%d) l %lu o %d"
-                          " t %ld H %d T %d X %ld %s %s\n",
+                          " t %lu H %d T %d X %lu %s %s\n",
                           innconf->artcutoff, innconf->maxconnections, count,
                           innconf->maxartsize, MaxOutgoing,
-                          (long) TimeOut.tv_sec, RemoteLimit, RemoteTotal,
-                          (long) RemoteTimer,
+                          (unsigned long) TimeOut.tv_sec, RemoteLimit,
+                          RemoteTotal, (unsigned long) RemoteTimer,
                           innconf->xrefslave ? "slave" : "normal",
                           AnyIncoming ? "any" : "specified");
 
@@ -975,9 +976,9 @@ CCname(char *av[])
 {
     CHANNEL *cp;
     int i, count;
-    const char *mode;
     char *p;
 
+    /* Write information for 1 given channel. */
     if (av[0][0] != '\0') {
         for (p = av[0]; *p != '\0'; p++) {
             if (!isdigit((unsigned char) *p)) {
@@ -988,54 +989,70 @@ CCname(char *av[])
         if (cp == NULL || cp->Type == CTfree) {
             return xstrdup(CCnochannel);
         }
-        buffer_sprintf(&CCreply, "0 %s", CHANname(cp));
+        buffer_set(&CCreply, "0 ", 2);
+        CCnameWriteInfo(&CCreply, cp);
+        buffer_append(&CCreply, "", 1);
         return CCreply.data;
     }
+
+    /* Write information for all channels. */
     buffer_set(&CCreply, "0 ", 2);
     for (count = 0, i = 0; (cp = CHANiter(&i, CTany)) != NULL;) {
         if (cp->Type == CTfree)
             continue;
         if (++count > 1)
             buffer_append(&CCreply, "\n", 1);
-        buffer_append_sprintf(&CCreply, "%s", CHANname(cp));
-        switch (cp->Type) {
-        case CTremconn:
-            buffer_append_sprintf(&CCreply, ":remconn::");
-            break;
-        case CTreject:
-            buffer_append_sprintf(&CCreply, ":reject::");
-            break;
-        case CTnntp:
-            mode = (cp->MaxCnx > 0 && cp->ActiveCnx == 0) ? "paused" : "";
-            buffer_append_sprintf(&CCreply, ":%s:%ld:%s",
-                                  cp->State == CScancel ? "cancel" : "nntp",
-                                  (long) (Now.tv_sec - cp->LastActive), mode);
-            break;
-        case CTlocalconn:
-            buffer_append_sprintf(&CCreply, ":localconn::");
-            break;
-        case CTcontrol:
-            buffer_append_sprintf(&CCreply, ":control::");
-            break;
-        case CTfile:
-            buffer_append_sprintf(&CCreply, "::");
-            break;
-        case CTexploder:
-            buffer_append_sprintf(&CCreply, ":exploder::");
-            break;
-        case CTprocess:
-            buffer_append_sprintf(&CCreply, ":");
-            break;
-        default:
-            buffer_append_sprintf(&CCreply, ":unknown::");
-            break;
-        }
+        CCnameWriteInfo(&CCreply, cp);
     }
     buffer_append(&CCreply, "", 1);
     return CCreply.data;
 }
 
 
+/*
+**  Write the expected channel information for cp in the buffer.
+*/
+static void
+CCnameWriteInfo(struct buffer *buffer, CHANNEL *cp)
+{
+    const char *mode;
+
+    buffer_append_sprintf(buffer, "%s", CHANname(cp));
+    switch (cp->Type) {
+    case CTremconn:
+        buffer_append_sprintf(buffer, ":remconn::");
+        break;
+    case CTreject:
+        buffer_append_sprintf(buffer, ":reject::");
+        break;
+    case CTnntp:
+        mode = (cp->MaxCnx > 0 && cp->ActiveCnx == 0) ? "paused" : "";
+        buffer_append_sprintf(
+            buffer, ":%s:%lu:%s", cp->State == CScancel ? "cancel" : "nntp",
+            (unsigned long) (Now.tv_sec - cp->LastActive), mode);
+        break;
+    case CTlocalconn:
+        buffer_append_sprintf(buffer, ":localconn::");
+        break;
+    case CTcontrol:
+        buffer_append_sprintf(buffer, ":control::");
+        break;
+    case CTfile:
+        buffer_append_sprintf(buffer, "::");
+        break;
+    case CTexploder:
+        buffer_append_sprintf(buffer, ":exploder::");
+        break;
+    case CTprocess:
+        buffer_append_sprintf(buffer, ":");
+        break;
+    default:
+        buffer_append_sprintf(buffer, ":unknown::");
+        break;
+    }
+}
+
+
 /*
 **  Create a newsgroup.
 */
@@ -1079,7 +1096,10 @@ CCnewgroup(char *av[])
     if (!is_valid_utf8(who))
         return "1 Invalid UTF-8 creator's name";
 
-    if (strlen(Name) + strlen(Rest) > SMBUF - 24)
+    /* Do not allow newsgroup names of arbitrary size.  There's no standardized
+     * limit except for the length of NNTP arguments, so let's take it even
+     * though it's a huge value for a newsgroup name. */
+    if (strlen(Name) > NNTP_MAXLEN_ARG || strlen(Rest) > NNTP_MAXLEN_ARG + 1)
         return "1 Name too long";
 
     if ((ngp = NGfind(Name)) != NULL)
@@ -1095,7 +1115,7 @@ CCnewgroup(char *av[])
         syslog(L_ERROR, "%s cant open %s %m", LogName, TIMES);
         IOError(WHEN, oerrno);
     } else {
-        xasprintf(&buff, "%s %ld %s\n", Name, (long) Now.tv_sec, who);
+        xasprintf(&buff, "%s %lu %s\n", Name, (unsigned long) Now.tv_sec, who);
         if (xwrite(fd, buff, strlen(buff)) < 0) {
             oerrno = errno;
             syslog(L_ERROR, "%s cant write %s %m", LogName, TIMES);
@@ -1182,8 +1202,9 @@ CCparam(char *av[])
         syslog(L_NOTICE, "%s changed -o %d", LogName, MaxOutgoing);
         break;
     case 't':
-        TimeOut.tv_sec = atol(p);
-        syslog(L_NOTICE, "%s changed -t %ld", LogName, (long) TimeOut.tv_sec);
+        TimeOut.tv_sec = atoll(p);
+        syslog(L_NOTICE, "%s changed -t %lu", LogName,
+               (unsigned long) TimeOut.tv_sec);
         break;
     case 'H':
         RemoteLimit = atoi(p);
@@ -1201,8 +1222,9 @@ CCparam(char *av[])
         RemoteTotal = temp;
         break;
     case 'X':
-        RemoteTimer = (time_t) atoi(p);
-        syslog(L_NOTICE, "%s changed -X %d", LogName, (int) RemoteTimer);
+        RemoteTimer = (time_t) atoll(p);
+        syslog(L_NOTICE, "%s changed -X %lu", LogName,
+               (unsigned long) RemoteTimer);
         break;
     }
     return NULL;
@@ -1844,7 +1866,8 @@ CCtrace(char *av[])
     case '7':
     case '8':
     case '9':
-        if ((cp = CHANfromdescriptor(atoi(p))) == NULL)
+        cp = CHANfromdescriptor(atoi(p));
+        if (cp == NULL || cp->Type == CTfree)
             return CCnochannel;
         CHANtracing(cp, Flag);
         break;
@@ -2136,6 +2159,14 @@ CCsetup(void)
     }
 #endif /* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 
+    if (!isvalidfd(i)) {
+        syslog(L_FATAL,
+               "%s cant CHANcreate %s: file descriptor %d too high (see "
+               "rlimitnofile in inn.conf)",
+               LogName, CCpath, i);
+        exit(1);
+    }
+
     CCchan = CHANcreate(i, CTcontrol, CSwaiting, CCreader, CCwritedone);
     syslog(L_NOTICE, "%s ccsetup %s", LogName, CHANname(CCchan));
     RCHANadd(CCchan);
