@@ -1,5 +1,4 @@
-/*  $Id: perl.c 10229 2018-01-28 21:21:21Z iulius $
-**
+/*
 **  Perl filtering support for innd.
 **
 **  Originally written by Christophe Wolfhugel <wolf@pasteur.fr> (although
@@ -29,23 +28,21 @@
 **     active.
 */
 
-#include "config.h"
+#include "portable/system.h"
 
 /* Skip this entire file if DO_PERL (./configure --with-perl) isn't set. */
 #if DO_PERL
 
-#include "clibrary.h"
-#include "inn/wire.h"
-#include "innd.h"
+#    include "inn/wire.h"
+#    include "innd.h"
 
-#include <EXTERN.h>
-#pragma GCC diagnostic ignored "-Wredundant-decls"
-#include <perl.h>
-#pragma GCC diagnostic warning "-Wredundant-decls"
-#include <XSUB.h>
-#include "ppport.h"
+#    include <EXTERN.h>
+#    include <perl.h>
 
-#include "innperl.h"
+#    include "ppport.h"
+#    include <XSUB.h>
+
+#    include "innperl.h"
 
 /* Prototypes for XS functions. */
 XS(XS_INN_addhist);
@@ -65,43 +62,49 @@ char *
 PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 {
     dSP;
-    const ARTHEADER * hp;
+    const ARTHEADER *hp;
     const HDRCONTENT *hc = data->HdrContent;
-    HV *        hdr;
-    CV *        filter;
-    int         i, rc;
-    char *      p;
+    HV *hdr;
+    CV *filter;
+    int i, rc;
+    char *p;
     static char buf[256];
-    bool        failure;
-    SV *        errsv;
+    bool failure;
+    SV *errsv;
 
-    if (!PerlFilterActive) return NULL;
+    if (!PerlFilterActive)
+        return NULL;
     filter = perl_get_cv("filter_art", 0);
-    if (!filter) return NULL;
+    if (!filter)
+        return NULL;
 
-    /* Create %hdr and stash a copy of every known header. */
+    /* Create %hdr and stash a copy of every known header field. */
     hdr = perl_get_hv("hdr", 1);
-    for (i = 0 ; i < MAX_ARTHEADER ; i++) {
-	if (HDR_FOUND(i)) {
-	    hp = &ARTheaders[i];
-            (void) hv_store(hdr, (char *) hp->Name, hp->Size, newSVpv(HDR(i), 0), 0);
-	}
+    for (i = 0; i < MAX_ARTHEADER; i++) {
+        if (HDR_FOUND(i)) {
+            hp = &ARTheaders[i];
+            (void) hv_store(hdr, (char *) hp->Name, hp->Size,
+                            newSVpv(HDR(i), 0), 0);
+        }
     }
 
     /* Store the article body.  We don't want to make another copy of it,
        since it could potentially be quite large.  In testing, this produced
        a 17% speed improvement over making a copy of the article body
        for a fairly heavy filter.
-       Available since Perl 5.7.1, newSVpvn_share allows to avoid such
+       Available since Perl 5.7.1, newSVpvn_share allows avoiding such
        a copy (getting round its use for older versions of Perl leads
        to unreliable SV * bodies as for regexps).  And for Perl not to
        compute a hash for artBody, we give it "42". */
     if (artBody) {
-#if (PERL_REVISION == 5) && ((PERL_VERSION < 7) || ((PERL_VERSION == 7) && (PERL_SUBVERSION < 1)))
+#    if (PERL_REVISION == 5)   \
+        && ((PERL_VERSION < 7) \
+            || ((PERL_VERSION == 7) && (PERL_SUBVERSION < 1)))
         (void) hv_store(hdr, "__BODY__", 8, newSVpv(artBody, artLen), 0);
-#else
-        (void) hv_store(hdr, "__BODY__", 8, newSVpvn_share(artBody, artLen, 42), 0);
-#endif /* Perl < 5.7.1 */
+#    else
+        (void) hv_store(hdr, "__BODY__", 8,
+                        newSVpvn_share(artBody, artLen, 42), 0);
+#    endif /* Perl < 5.7.1 */
     }
 
     (void) hv_store(hdr, "__LINES__", 9, newSViv(lines), 0);
@@ -110,7 +113,7 @@ PLartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
     SAVETMPS;
     PUSHMARK(SP);
     PUTBACK;
-    rc = perl_call_sv((SV *) filter, G_EVAL|G_SCALAR|G_NOARGS);
+    rc = perl_call_sv((SV *) filter, G_EVAL | G_SCALAR | G_NOARGS);
     SPAGAIN;
 
     hv_undef(hdr);
@@ -153,16 +156,18 @@ char *
 PLmidfilter(char *messageID)
 {
     dSP;
-    CV          *filter;
-    int         rc;
-    char        *p;
+    CV *filter;
+    int rc;
+    char *p;
     static char buf[256];
-    bool        failure;
-    SV *        errsv;
+    bool failure;
+    SV *errsv;
 
-    if (!PerlFilterActive) return NULL;
+    if (!PerlFilterActive)
+        return NULL;
     filter = perl_get_cv("filter_messageid", 0);
-    if (!filter) return NULL;
+    if (!filter)
+        return NULL;
 
     /* Pass filter_messageid() the message ID on the Perl stack. */
     ENTER;
@@ -170,7 +175,7 @@ PLmidfilter(char *messageID)
     PUSHMARK(SP);
     XPUSHs(sv_2mortal(newSVpv(messageID, 0)));
     PUTBACK;
-    rc = perl_call_sv((SV *) filter, G_EVAL|G_SCALAR);
+    rc = perl_call_sv((SV *) filter, G_EVAL | G_SCALAR);
     SPAGAIN;
 
     /* Check $@, which will be set if the sub died. */
@@ -206,28 +211,29 @@ PLmidfilter(char *messageID)
 **  mode and the reason.
 */
 void
-PLmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
+PLmode(OPERATINGMODE CurrentMode, OPERATINGMODE NewMode, char *reason)
 {
     dSP;
-    HV          *mode;
-    CV          *filter;
-    bool        failure;
-    SV *        errsv;
+    HV *mode;
+    CV *filter;
+    bool failure;
+    SV *errsv;
 
     filter = perl_get_cv("filter_mode", 0);
-    if (!filter) return;
+    if (!filter)
+        return;
 
     /* Current mode goes into $mode{Mode}, new mode in $mode{NewMode}, and
        the reason in $mode{reason}. */
     mode = perl_get_hv("mode", 1);
 
-    if (Mode == OMrunning)
+    if (CurrentMode == OMrunning)
         (void) hv_store(mode, "Mode", 4, newSVpv("running", 7), 0);
-    if (Mode == OMpaused)
+    if (CurrentMode == OMpaused)
         (void) hv_store(mode, "Mode", 4, newSVpv("paused", 6), 0);
-    if (Mode == OMthrottled)
+    if (CurrentMode == OMthrottled)
         (void) hv_store(mode, "Mode", 4, newSVpv("throttled", 9), 0);
-    if (Mode == OMshutdown)
+    if (CurrentMode == OMshutdown)
         (void) hv_store(mode, "Mode", 4, newSVpv("shutdown", 8), 0);
 
     if (NewMode == OMrunning)
@@ -246,7 +252,7 @@ PLmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
     PUSHMARK(SP);
     PUTBACK;
 
-    perl_call_sv((SV *) filter, G_EVAL|G_DISCARD|G_NOARGS);
+    perl_call_sv((SV *) filter, G_EVAL | G_DISCARD | G_NOARGS);
 
     SPAGAIN;
 
@@ -255,7 +261,7 @@ PLmode(OPERATINGMODE Mode, OPERATINGMODE NewMode, char *reason)
     if (SvTRUE(errsv)) {
         failure = true;
         syslog(L_ERROR, "Perl function filter_mode died: %s",
-                SvPV(errsv, PL_na));
+               SvPV(errsv, PL_na));
         (void) POPs;
     } else {
         failure = false;
@@ -282,7 +288,7 @@ char *
 PLstats(void)
 {
     dSP;
-    char *argv[] = { NULL };
+    char *argv[] = {NULL};
 
     if (perl_get_cv("filter_stats", false) == NULL)
         return NULL;
@@ -290,17 +296,17 @@ PLstats(void)
         char *stats = NULL;
         char *result;
 
-	ENTER;
-	SAVETMPS;
+        ENTER;
+        SAVETMPS;
         /* No need for PUSHMARK(SP) with call_argv(). */
-	perl_call_argv("filter_stats", G_EVAL | G_NOARGS, argv);
-	SPAGAIN;
+        perl_call_argv("filter_stats", G_EVAL | G_NOARGS, argv);
+        SPAGAIN;
         result = POPp;
         if (result != NULL && *result)
             stats = xstrdup(result);
-	PUTBACK;
-	FREETMPS;
-	LEAVE;
+        PUTBACK;
+        FREETMPS;
+        LEAVE;
 
         return stats;
     }
@@ -325,12 +331,9 @@ PLstats(void)
 XS(XS_INN_addhist)
 {
     dXSARGS;
-    int         i;
-    char        tbuff[32];
-    char*       parambuf[6];
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    int i;
+    char tbuff[32];
+    char *parambuf[6];
 
     if (items < 1 || items > 5)
         croak("Usage INN::addhist(msgid,[arrival,articletime,expire,token])");
@@ -340,7 +343,7 @@ XS(XS_INN_addhist)
 
     /* If any of the times are missing, they should default to now. */
     if (i < 4) {
-        snprintf(tbuff, sizeof(tbuff), "%ld", (long) time(NULL));
+        snprintf(tbuff, sizeof(tbuff), "%lu", (unsigned long) time(NULL));
         for (; i < 4; i++)
             parambuf[i] = tbuff;
     }
@@ -368,25 +371,24 @@ XS(XS_INN_addhist)
 XS(XS_INN_article)
 {
     dXSARGS;
-    char *      msgid;
-    TOKEN       token;
-    ARTHANDLE * art;
-    char *      p;
-    size_t      len;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    char *msgid;
+    TOKEN token;
+    ARTHANDLE *art;
+    char *p;
+    size_t len;
 
     if (items != 1)
-	croak("Usage: INN::article(msgid)");
+        croak("Usage: INN::article(msgid)");
 
     /* Get the article token from the message ID and the history file. */
     msgid = (char *) SvPV(ST(0), PL_na);
-    if (!HISlookup(History, msgid, NULL, NULL, NULL, &token)) XSRETURN_UNDEF;
+    if (!HISlookup(History, msgid, NULL, NULL, NULL, &token))
+        XSRETURN_UNDEF;
 
     /* Retrieve the article and convert it from wire format. */
     art = SMretrieve(token, RETR_ALL);
-    if (art == NULL) XSRETURN_UNDEF;
+    if (art == NULL)
+        XSRETURN_UNDEF;
     p = wire_to_native(art->data, art->len, &len);
     SMfreearticle(art);
 
@@ -405,11 +407,8 @@ XS(XS_INN_article)
 XS(XS_INN_cancel)
 {
     dXSARGS;
-    char        *msgid;
-    char        *parambuf[2];
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    char *msgid;
+    char *parambuf[2];
 
     if (items != 1)
         croak("Usage: INN::cancel(msgid)");
@@ -434,11 +433,8 @@ XS(XS_INN_cancel)
 XS(XS_INN_filesfor)
 {
     dXSARGS;
-    char        *msgid;
-    TOKEN       token;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    char *msgid;
+    TOKEN token;
 
     if (items != 1)
         croak("Usage: INN::filesfor(msgid)");
@@ -458,10 +454,7 @@ XS(XS_INN_filesfor)
 XS(XS_INN_havehist)
 {
     dXSARGS;
-    char        *msgid;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    char *msgid;
 
     if (items != 1)
         croak("Usage: INN::havehist(msgid)");
@@ -475,37 +468,36 @@ XS(XS_INN_havehist)
 
 
 /*
-**  Takes the message ID of an article and returns the article headers as
+**  Takes the Message-ID of an article and returns the article headers as
 **  a string or undef if the article wasn't found.  Each line of the header
-**  will end with \n.
+**  field body will end with \n.
 */
 XS(XS_INN_head)
 {
     dXSARGS;
-    char *      msgid;
-    TOKEN       token;
-    ARTHANDLE * art;
-    char *      p;
-    size_t      len;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    char *msgid;
+    TOKEN token;
+    ARTHANDLE *art;
+    char *p;
+    size_t len;
 
     if (items != 1)
         croak("Usage: INN::head(msgid)");
 
-    /* Get the article token from the message ID and the history file. */
+    /* Get the article token from the Message-ID and the history file. */
     msgid = (char *) SvPV(ST(0), PL_na);
-    if (!HISlookup(History, msgid, NULL, NULL, NULL, &token)) XSRETURN_UNDEF;
+    if (!HISlookup(History, msgid, NULL, NULL, NULL, &token))
+        XSRETURN_UNDEF;
 
-    /* Retrieve the article header and convert it from wire format. */
+    /* Retrieve the article headers and convert them from wire format. */
     art = SMretrieve(token, RETR_HEAD);
-    if (art == NULL) XSRETURN_UNDEF;
+    if (art == NULL)
+        XSRETURN_UNDEF;
     p = wire_to_native(art->data, art->len, &len);
     SMfreearticle(art);
 
-    /* Push a copy of the article header onto the Perl stack, free our
-       temporary memory allocation, and return the header to Perl. */
+    /* Push a copy of the article headers onto the Perl stack, free our
+       temporary memory allocation, and return the headers to Perl. */
     ST(0) = sv_2mortal(newSVpv(p, len));
     free(p);
     XSRETURN(1);
@@ -519,13 +511,10 @@ XS(XS_INN_head)
 XS(XS_INN_newsgroup)
 {
     dXSARGS;
-    char *      newsgroup;
-    NEWSGROUP * ngp;
-    char *      end;
-    int         size;
-
-    /* Suppress warnings for the mandatory XS argument. */
-    cv = cv;
+    char *newsgroup;
+    NEWSGROUP *ngp;
+    char *end;
+    int size;
 
     if (items != 1)
         croak("Usage: INN::newsgroup(group)");
