@@ -107,7 +107,7 @@ static const struct config config_table[] = {
     {K(pathspool),                  STRING(NULL)      },
     {K(pathtmp),                    STRING(NULL)      },
 
- /* The following settings are specific to innd. */
+    /* The following settings are specific to innd. */
     {K(artcutoff),                  UNUMBER(10)       },
     {K(badiocount),                 UNUMBER(5)        },
     {K(bindaddress),                STRING(NULL)      },
@@ -150,7 +150,7 @@ static const struct config config_table[] = {
     {K(wipexpire),                  UNUMBER(10)       },
     {K(xrefslave),                  BOOL(false)       },
 
- /* The following settings are specific to nnrpd. */
+    /* The following settings are specific to nnrpd. */
     {K(addinjectiondate),           BOOL(true)        },
     {K(addinjectionpostingaccount), BOOL(false)       },
     {K(addinjectionpostinghost),    BOOL(true)        },
@@ -198,11 +198,11 @@ static const struct config config_table[] = {
     {K(tlsprotocols),               LIST(NULL)        },
 #endif  /* HAVE_OPENSSL */
 
-  /* The following settings are used by nnrpd and rnews. */
+    /* The following settings are used by nnrpd and rnews. */
     {K(nnrpdposthost),              STRING(NULL)      },
     {K(nnrpdpostport),              UNUMBER(119)      },
 
- /* The following settings are specific to the storage subsystem. */
+    /* The following settings are specific to the storage subsystem. */
     {K(articlemmap),                BOOL(true)        },
     {K(cnfscheckfudgesize),         UNUMBER(0)        },
     {K(immediatecancel),            BOOL(false)       },
@@ -210,21 +210,22 @@ static const struct config config_table[] = {
     {K(nfswriter),                  BOOL(false)       },
     {K(nnrpdcheckart),              BOOL(true)        },
     {K(overcachesize),              UNUMBER(128)      },
+    {K(ovflushcount),               UNUMBER(50)       },
     {K(ovgrouppat),                 STRING(NULL)      },
     {K(storeonxref),                BOOL(true)        },
     {K(tradindexedmmap),            BOOL(true)        },
     {K(useoverchan),                BOOL(false)       },
     {K(wireformat),                 BOOL(true)        },
 
- /* The following settings are specific to the history subsystem. */
+    /* The following settings are specific to the history subsystem. */
     {K(hismethod),                  STRING(NULL)      },
 
- /* The following settings are specific to rc.news. */
+    /* The following settings are specific to rc.news. */
     {K(docnfsstat),                 BOOL(false)       },
     {K(innflags),                   STRING(NULL)      },
     {K(pgpverify),                  BOOL(false)       },
 
- /* The following settings are specific to innwatch. */
+    /* The following settings are specific to innwatch. */
     {K(doinnwatch),                 BOOL(true)        },
     {K(innwatchbatchspace),         UNUMBER(4000)     },
     {K(innwatchlibspace),           UNUMBER(25000)    },
@@ -235,11 +236,11 @@ static const struct config config_table[] = {
     {K(innwatchspoolnodes),         UNUMBER(200)      },
     {K(innwatchspoolspace),         UNUMBER(25000)    },
 
- /* The following settings are specific to scanlogs.
-  * Keep a low value by default as it has a privacy impact; three days
-  * should be enough to diagnose the most common cases of malfunction
-  * and abuse.  Where more is needed, it is easy for the news admin
-  * to increase it. */
+    /* The following settings are specific to scanlogs.
+     * Keep a low value by default as it has a privacy impact; three days
+     * should be enough to diagnose the most common cases of malfunction
+     * and abuse.  Where more is needed, it is easy for the news admin
+     * to increase it. */
     {K(logcycles),                  UNUMBER(3)        },
 };
 
@@ -440,14 +441,27 @@ static bool
 innconf_validate(struct config_group *group)
 {
     bool okay = true;
+    unsigned int i;
     char *fqdn;
 
+    /* Checks to ensure valid Message-IDs will be generated. */
     fqdn = inn_getfqdn(innconf->domain);
     if (fqdn == NULL) {
         warn("hostname does not resolve or domain not set in inn.conf");
         okay = false;
     }
+    if (innconf->domain != NULL && !IsValidDomain(innconf->domain)) {
+        warn("domain in inn.conf contains invalid characters not suitable for "
+             "Message-IDs");
+        okay = false;
+    }
+    if (innconf->domain == NULL && !IsValidDomain(fqdn)) {
+        warn("the FQDN of the server contains invalid characters not suitable "
+             "for Message-IDs");
+        okay = false;
+    }
     free(fqdn);
+
     if (innconf->mta == NULL) {
         warn("must set mta in inn.conf");
         okay = false;
@@ -480,9 +494,10 @@ innconf_validate(struct config_group *group)
         okay = false;
     }
 
+    /* Warn (but do not bail out) if the generation of keywords is active
+     * without being advertised in overview data. */
     if (innconf->keywords) {
         bool found = false;
-        unsigned int i;
         if (innconf->extraoverviewadvertised->strings != NULL) {
             for (i = 0; i < innconf->extraoverviewadvertised->count; i++) {
                 if (innconf->extraoverviewadvertised->strings[i] != NULL
@@ -515,6 +530,70 @@ innconf_validate(struct config_group *group)
         }
     }
 
+    /* Do not accept deprecated Bytes and Lines header fields in overview data
+     * as they are internally treated as metadata items. */
+    if (innconf->extraoverviewadvertised->strings != NULL) {
+        for (i = 0; i < innconf->extraoverviewadvertised->count; i++) {
+            if (innconf->extraoverviewadvertised->strings[i] != NULL
+                && (strcasecmp(innconf->extraoverviewadvertised->strings[i],
+                               "Bytes")
+                        == 0
+                    || strcasecmp(innconf->extraoverviewadvertised->strings[i],
+                                  "Lines")
+                           == 0)) {
+                config_error_param(
+                    group, "extraoverviewadvertised",
+                    "Bytes and Lines not allowed in overview data");
+                okay = false;
+                break;
+            }
+        }
+    }
+    if (innconf->extraoverviewhidden->strings != NULL) {
+        for (i = 0; i < innconf->extraoverviewhidden->count; i++) {
+            if (innconf->extraoverviewhidden->strings[i] != NULL
+                && (strcasecmp(innconf->extraoverviewhidden->strings[i],
+                               "Bytes")
+                        == 0
+                    || strcasecmp(innconf->extraoverviewhidden->strings[i],
+                                  "Lines")
+                           == 0)) {
+                config_error_param(
+                    group, "extraoverviewhidden",
+                    "Bytes and Lines not allowed in overview data");
+                okay = false;
+                break;
+            }
+        }
+    }
+
+    /* Do not accept header field names beginning with a dot (extra complexity
+     * not handled in the code for correctly dot-stuffing them). */
+    if (innconf->extraoverviewadvertised->strings != NULL) {
+        for (i = 0; i < innconf->extraoverviewadvertised->count; i++) {
+            if (innconf->extraoverviewadvertised->strings[i] != NULL
+                && (*innconf->extraoverviewadvertised->strings[i] == '.')) {
+                config_error_param(group, "extraoverviewadvertised",
+                                   "Header field names beginning with a dot "
+                                   "not allowed in overview data");
+                okay = false;
+                break;
+            }
+        }
+    }
+    if (innconf->extraoverviewhidden->strings != NULL) {
+        for (i = 0; i < innconf->extraoverviewhidden->count; i++) {
+            if (innconf->extraoverviewhidden->strings[i] != NULL
+                && (*innconf->extraoverviewhidden->strings[i] == '.')) {
+                config_error_param(group, "extraoverviewhidden",
+                                   "Header field names beginning with a dot "
+                                   "not allowed in overview data");
+                okay = false;
+                break;
+            }
+        }
+    }
+
     return okay;
 }
 
@@ -844,7 +923,7 @@ innconf_compare(struct innconf *conf1, struct innconf *conf2)
             }
             break;
         default:
-            die("internal error: invalid type in row %d of config table", i);
+            die("internal error: invalid type in row %u of config table", i);
         }
     return okay;
 }
