@@ -5,23 +5,28 @@
 **  the server functions are implemented; there is no client support as yet.
 */
 
-#include "config.h"
-#include "clibrary.h"
+#include "portable/system.h"
+
 #include "portable/socket.h"
 #include <ctype.h>
 #include <errno.h>
 
 #ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
+#    include <sys/time.h>
 #endif
 #include <time.h>
 
+/* Needed on AIX 4.1 to get fd_set and friends. */
+#if HAVE_SYS_SELECT_H
+#    include <sys/select.h>
+#endif
+
 #include "inn/buffer.h"
 #include "inn/innconf.h"
+#include "inn/libinn.h"
 #include "inn/network.h"
 #include "inn/nntp.h"
 #include "inn/vector.h"
-#include "inn/libinn.h"
 
 /* State for an NNTP connection. */
 struct nntp {
@@ -370,9 +375,9 @@ nntp_respond(struct nntp *nntp, enum nntp_code code, const char *format, ...)
     va_list args;
 
     if (format == NULL)
-        buffer_append_sprintf(&nntp->out, "%d\r\n", code);
+        buffer_append_sprintf(&nntp->out, "%u\r\n", code);
     else {
-        buffer_append_sprintf(&nntp->out, "%d ", code);
+        buffer_append_sprintf(&nntp->out, "%u ", code);
         va_start(args, format);
         buffer_append_vsprintf(&nntp->out, format, args);
         va_end(args);
@@ -393,9 +398,9 @@ nntp_respond_noflush(struct nntp *nntp, enum nntp_code code,
     va_list args;
 
     if (format == NULL)
-        buffer_append_sprintf(&nntp->out, "%d\r\n", code);
+        buffer_append_sprintf(&nntp->out, "%u\r\n", code);
     else {
-        buffer_append_sprintf(&nntp->out, "%d ", code);
+        buffer_append_sprintf(&nntp->out, "%u ", code);
         va_start(args, format);
         buffer_append_vsprintf(&nntp->out, format, args);
         va_end(args);
