@@ -781,7 +781,7 @@ groupid_new(group_id_t *gno, DB_TXN *tid)
     }
 
     if (val.size % sizeof(group_id_t)) {
-        warn("OVDB: invalid size (%d) for !groupid_freelist", val.size);
+        warn("OVDB: invalid size (%u) for !groupid_freelist", val.size);
         return EINVAL;
     }
 
@@ -830,7 +830,7 @@ groupid_free(group_id_t gno, DB_TXN *tid)
     }
 
     if (val.size % sizeof(group_id_t)) {
-        warn("OVDB: invalid size (%d) for !groupid_freelist", val.size);
+        warn("OVDB: invalid size (%u) for !groupid_freelist", val.size);
         return EINVAL;
     }
 
@@ -1280,10 +1280,14 @@ count_records(struct groupinfo *gi)
         ret = cursor->c_get(cursor, &key, &val, DB_NEXT);
     }
     cursor->c_close(cursor);
-    if (gi->count == 0)
-        gi->low = gi->high + 1;
-    else
+    if (gi->count == 0) {
+        gi->low = gi->low > gi->high ? gi->low : gi->high;
+        if (gi->low == 0)
+            gi->low = 1;
+        gi->high = gi->low - 1;
+    } else {
         gi->low = newlow;
+    }
 
     if (ret == DB_NOTFOUND)
         return 0;
@@ -1413,7 +1417,7 @@ ovdb_check_user(void)
         int rv;
         uid_t uid;
 
-        rv = get_news_uid_gid(&uid, false, false);
+        rv = get_news_uid_gid(&uid, NULL, false);
 
         if (rv != 0) {
             syswarn("OVDB: can't resolve runasuser user to a UID");
@@ -1474,7 +1478,7 @@ check_version(void)
         memcpy(&dv, val.data, sizeof dv);
 
     if (dv > DATA_VERSION_COMPRESS) {
-        warn("OVDB: can't open database: unknown version %d", dv);
+        warn("OVDB: can't open database: unknown version %u", dv);
         vdb->close(vdb, 0);
         return EINVAL;
     }
@@ -2433,7 +2437,7 @@ ovdb_search(void *handle, ARTNUM *artnum, char **data, int *len, TOKEN *token,
         memcpy(&sz, dp, sizeof(uint32_t));
         sz = ntohl(sz);
         if (sz > MAX_UNZIP_SZ) { /* sanity check */
-            warn("OVDB: search: bogus sz: %d", sz);
+            warn("OVDB: search: bogus sz: %u", sz);
             sz = 0;
         }
     }
@@ -2958,8 +2962,12 @@ ovdb_expiregroup(const char *group, int *lo, struct history *h)
 
             gi.status &= ~GROUPINFO_EXPIRING;
             gi.expired = time(NULL);
-            if (gi.count == 0 && lowest == 0)
-                gi.low = gi.high + 1;
+            if (gi.count == 0 && lowest == 0) {
+                gi.low = gi.low > gi.high ? gi.low : gi.high;
+                if (gi.low == 0)
+                    gi.low = 1;
+                gi.high = gi.low - 1;
+            }
         }
 
         ret = groupinfo->put(groupinfo, tid, &gkey, &gval, 0);
